@@ -9,6 +9,7 @@
 # Brandon Bailey <Twitter: @ge0stigm4> (Co-designer of original concept) 
 # Bob Harold (Guidance on DNS)
 # Neamen Negash <nnegash at umich.edu> (Installer)
+# afreudenreich (Multiple bug fixes and enhancements)
 #
 # WebStor uses Wappalyzer's technologies database for pre-populated, name- 
 # indexed technology lookups against WebStor's stored responses.  Wappalyzer
@@ -74,30 +75,113 @@ from gevent import monkey
 
 orig_connect = urllib3.connection.HTTPConnection.connect
 
+parser = argparse.ArgumentParser()
+parser.add_argument("--ADD-HTTP-PORT", "-a", dest="HttpPortToAdd", default=None, help="Add a custom HTTP port.")
+parser.add_argument("--CLEAR-HTTP", "-aC", dest="ClearHttpPorts", default=False, action="store_true", \
+                    help="Clear any custom HTTP ports and revert to default of 80.")
+parser.add_argument("--ADD-HTTPS-PORT", "-b", dest="HttpsPortToAdd", default=None, help="Add a custom HTTPS port.")
+parser.add_argument("--CLEAR-HTTPS", "-bC", dest="ClearHttpsPorts", default=False, action="store_true", \
+                    help="Clear any custom HTTPS ports and revert to default of 443.")
+parser.add_argument("--ADD-CUSTOM-FINGERPRINT", "-c", dest="Fingerprint", default=None, \
+                    help="Add a custom fingerprint in the form <Name>,<RegEx>.")
+parser.add_argument("--DELETE-CUSTOM-FINGERPRINT", "-cD", dest="FingerprintNameToDelete", default=None, \
+                    help="Delete a custom fingerprint by name.")
+parser.add_argument("--IMPORT-CUSTOM-FINGERPRINT", "-cI", dest="ImportFingerprintFile", default=None, \
+                    help="Import a custom fingerprint file with the path specified.")
+parser.add_argument("--CLEAR-CUSTOM-FINGERPRINTS", "-cC", dest="ClearFingerprints", default=False, action="store_true", \
+                    help="Clears all custom fingerprints stored in DB.")
+parser.add_argument("--SHOW-CONFIG", "-g", dest="ShowConfigBrief", default=False, action="store_true", \
+                    help="Show current WebStor configuration (brief).")
+parser.add_argument("--SHOW-CONFIG-FULL", "-gF", dest="ShowConfigFull", default=False, action="store_true", \
+                    help="Show current WebStor configuration (full).")
+parser.add_argument("--RUN-MASSCAN", "-m", dest='ForceScan', default=False, action='store_true', \
+                    help="Runs a new port scan with Masscan on all configured TCP ports for HTTP and HTTPS, " \
+                    "against all configured ranges and any IP addresses from DNS records that are outside those ranges.")
+parser.add_argument("--SET-MASSCAN-RANGES", "-mR", dest="SetScanRanges", default=None, \
+                    help="Scan range or ranges, replaces existing ranges in DB, comma " \
+                    "separated, such as: -s 10.10.0.0/16,10.13.0.0/16,192.168.1.0/24")
+parser.add_argument("--ADD-RANGE", "-mA", dest="RangeToAdd", default=None, help="Add scan range.")
+parser.add_argument("--DELETE-RANGE", "-mD", dest="RangeToDelete", default=None, help="Delete scan range.")
+parser.add_argument("--IMPORT-MASSCAN-RANGES", "-mI", dest="ImportScanRanges", default=None, \
+                    help="Import scan ranges (CIDR blocks) from a specified file.")
+parser.add_argument("--ADD-PATH", "-p", dest="PathToAdd", default=None, help="Add paths for which to request " \
+                    "and store responses besides '/'.")
+parser.add_argument("--DELETE-PATH", "-pD", dest="PathToDelete", default=None, help="Delete paths for which to " \
+                    "request and store responses besides '/'.")
+parser.add_argument("--CLEAR-PATHS", "-pC", dest="ClearPaths", default=False, action="store_true", \
+                    help="Clear any custom URL request paths and revert to default of '/'.")
+parser.add_argument("--REFRESH-RESPONSES", "-r", dest="RefreshResponses", default=False, action="store_true", \
+                    help="Refresh URL responses in DB.")
+parser.add_argument("--SEARCH-PATTERN", "-sP", dest="SearchPattern", default=None, \
+                    help="Search for string or regular expression in WebStor database.")
+parser.add_argument("--SEARCH-CUSTOM-FINGERPRINT", "-sC", dest="SearchFingerprint", default=None, \
+                    help="Search for technology by name of user-provided custom fingerprint.")
+parser.add_argument("--SEARCH-WAPPALYZER", "-sW", dest="SearchWappalyzer", default=None, \
+                    help="Search for technology by name (from Wappalyzer Tech DB) in WebStor DB.")
+parser.add_argument("--NO-TSIG-KEY", "-tN", dest="UseTSIG", default=True, action="store_false", \
+                    help="Do not use DNSSec TSIG key stored in database or a file, even if present.")
+parser.add_argument("--TSIG-KEY-IMPORT", "-tI", dest="ImportTSIGFile", default=None, \
+                    help="Import a specified TSIG key file into the database")
+parser.add_argument("--TSIG-KEY-REPLACE", "-tR", dest="ReplacementTSIGFile", default=None, \
+                    help="Replace a TSIG key in the database with a specified file")
+parser.add_argument("--DELETE-TSIG", "-dT", dest="TSIGToDelete", default=None, \
+                    help="Delete a TSIG key from the database by name.")
+parser.add_argument("--USE-TSIG-FILE-ONLY", "-tF", dest="UseTSIGFileOnly", default=None, \
+                    help="Only use tsig file specified (full path), do not use TSIGs stored in the DB. " \
+                    "Applies to all domains, limiting WebStor to one TSIG for zone transfers in the current execution.")
+parser.add_argument("--DOWNLOAD-NEW-WAPPALYZER", '-w', dest="DLWap", default=False, action="store_true", \
+                    help="Download a new Wappalyzer fingerprints file directly from GitHub. Overwrites existing " \
+                    "Wappalyzer fingerprint data.")
+parser.add_argument("--LIST-WAPPALYZER-TECH-NAMES", "-wL", dest="ListWappalyzer", default=False, action="store_true", \
+                    help="List the names of all Wappalyzer technologies in the database.")
+parser.add_argument("--ZONE-XFER", "-z", dest='PerformZoneXfer', default=False, action='store_true', \
+                    help="Forces a new zone transfer using all domains, servers, and associated TSIG keys in DB")
+parser.add_argument("--ADD-DOMAIN", "-zA", dest="DomainDetails", default=None, \
+                    help="Add a domain in the form <Domain name>,<Server>,<TSIG Key Name>.")
+parser.add_argument("--DELETE-DOMAIN", "-zD", dest="DomainToDelete", default=None, \
+                    help="Delete a DNS domain from the database by name.")
+parser.add_argument("--IMPORT-ZONE-FILE", "-zI", dest="ImportZoneFile", default=None, \
+                    help="Add domains for zone transfers from a file.")
+parser.add_argument("--CLEAR-DOMAINS", "-zC", dest="ClearDomains", default=False, action="store_true", \
+                    help="Clears all DNS domains stored in DB.")
+parser.add_argument("--LIST-DOMAINS", "-zL", dest="ListDomains", default=False, action="store_true", \
+                    help="Lists all DNS domains stored in DB.")
+parser.add_argument("--LIST-OUTSIDE", "-e", dest="ListOutside", default=False, action="store_true", \
+                    help="Prints a list of all names and IPs from our zone transfers that are outside defined net ranges.")
+parser.add_argument("--SQL-CREDS", "-q", dest="SQLCredsFile", default=None, \
+                    help="Use SQL credentials in file at specified path.")
+
+args = parser.parse_args()
+
 sMySQLhost="localhost"
 sMySQLuser="root"
 sMySQLpw=""
 
-if os.path.exists('./ws-sql.txt'):
-    try:
-        MySQLcredFile = open('./ws-sql.txt', 'r')
-        lMySQLlines = MySQLcredFile.readlines()
-        sMySQLhost = lMySQLlines[0].rstrip()
-        sMySQLuser = lMySQLlines[1].rstrip()
-        sMySQLpw = lMySQLlines[2].rstrip()
-        mysqlconn = mysql.connector.connect(host=sMySQLhost, user=sMySQLuser, password=sMySQLpw)
-    except Exception as e:
-        print("Error reading MySQL credential file or connecting to database. Falling back to default credentials." \
-              " Error was: %s\n" % e)
-        sMySQLhost="localhost"
-        sMySQLuser="root"
-        sMySQLpw=""
+if args.SQLCredsFile != None:
+    if os.path.exists(args.SQLCredsFile):
         try:
+            MySQLcredFile = open(args.SQLCredsFile, 'r')
+            lMySQLlines = MySQLcredFile.readlines()
+            sMySQLhost = lMySQLlines[0].rstrip()
+            sMySQLuser = lMySQLlines[1].rstrip()
+            sMySQLpw = lMySQLlines[2].rstrip()
             mysqlconn = mysql.connector.connect(host=sMySQLhost, user=sMySQLuser, password=sMySQLpw)
         except Exception as e:
-            print("Failed after falling back to default credentials. Exiting. Error was: %s\n\n" \
-                   % e)
-            exit(1)
+            print("Error reading MySQL credential file or connecting to database. Falling back to default credentials." \
+                  " Error was: %s\n" % e)
+            sMySQLhost="localhost"
+            sMySQLuser="root"
+            sMySQLpw=""
+            try:
+                mysqlconn = mysql.connector.connect(host=sMySQLhost, user=sMySQLuser, password=sMySQLpw)
+            except Exception as e:
+                print("Failed after falling back to default credentials. Exiting. Error was: %s\n\n" \
+                      % e)
+                exit(1)
+    else:
+        print("Specified MySQL credential file does not exist. Exiting.")
+        exit(1)
+    
 else:
     try:
         mysqlconn = mysql.connector.connect(host=sMySQLhost, user=sMySQLuser, password=sMySQLpw)
@@ -105,7 +189,7 @@ else:
         print("Failed connecting with MySQL default credentials. Exiting. Error was: %s\n\n" \
               "Do you have MariaDB 10.0.5 or newer installed?" % e)
         exit(1)
-
+    
 cursor = mysqlconn.cursor()
 
 
@@ -763,6 +847,22 @@ def import_scan_ranges(sFileName):
     return
 
 
+def add_scan_range(sRangeToAdd):
+    print("Attempting to add scan range: %s" % sRangeToAdd)
+    sInsertRange = """INSERT IGNORE INTO target_ranges (cidr) VALUES (%s)"""
+    lCIDRblock = sRangeToAdd.split('/')
+    if (len(lCIDRblock) == 2) and (is_valid_ipv4(lCIDRblock[0])) and (int(lCIDRblock[1])>7 and int(lCIDRblock[1])<33):
+        try:
+            cursor.execute(sInsertRange, (sRangeToAdd.rstrip(),)) 
+            mysqlconn.commit()
+            print("Successfully added scan range into database.")
+        except Exception as e:
+            print("Error inserting scan ranges into database: %s", e)
+    else:
+        print("Not adding invalid CIDR block: %s" % sRangeToAdd )
+    return
+
+
 def delete_scan_range(sRangeToRemove):
     print("Attempting to delete scan range: %s" % sRangeToRemove)
     sql_delete_range = """DELETE FROM target_ranges WHERE cidr = %s""" 
@@ -791,26 +891,22 @@ def import_tsig(sFileName):
     try:
         TSIGFile = open(sFileName, 'r') 
         aTSIGLines = TSIGFile.readlines()
-        if aTSIGLines[0].startswith("key"):
-            sKey = aTSIGLines[0].split()[1]
-            print(sKey)
-        else:
-            print("Invalid key name in TSIG file")
-            return
-        if aTSIGLines[1].find("algorithm"):
-            sAlgorithm = aTSIGLines[1].split('"')[1]
-            print(sAlgorithm)
-        else:
-            print("Invalid algorithm in TSIG file")
-            return
-        if aTSIGLines[2].find("secret"):
-            sSecret = aTSIGLines[2].split('"')[1]
-            print(sSecret)
-        else:
-            print("Invalid secret in TSIG file")
-            return
+        for aTSIGLine in aTSIGLines:
+            if aTSIGLine.startswith("key"):
+                sKey = aTSIGLine.split()[1]
+                print(sKey)
+            elif aTSIGLine.lstrip().startswith("algorithm"):
+                sAlgorithm = aTSIGLine.split('"')[1]
+                print(sAlgorithm)
+            elif aTSIGLine.lstrip().startswith("secret"):
+                sSecret = aTSIGLine.split('"')[1]
+                print("*****")
     except Exception as e:
-        print("Error importing TSIG from file: %s", e)
+        print(f"Error importing TSIG from file: {e}")
+        return
+
+    if not (sKey and sAlgorithm and sSecret):
+        print("TSIG key information missing")
         return
 
     sInsertTsig = """INSERT INTO tsig (name, algorithm, secret) VALUES (%s,%s,%s)""" 
@@ -947,22 +1043,38 @@ def add_domain(sDomainToAdd):
 def download_wappalyzer():
     print("Downloading Wappalyzer Fingerprints from GitHub Repo.")
     sWappalyzerResponse = ""
-    try:
-        r = requests.get("https://raw.githubusercontent.com/AliasIO/wappalyzer/master/src/technologies.json", \
-                         timeout=5)
-        sWappalyzerResponse = str(r.text)
-    except:
-        print("Wappalyzer technologies json file not available.")
 
+    #Wappalyzer has divided the former technologies.json into many files in a technologies dir.  
+    #We'll take all json files from it.
+    try:
+        rTechFolder = requests.get("https://api.github.com/repos/AliasIO/wappalyzer/contents/src/technologies?ref=master", \
+                      timeout=5).json()
+    except:
+        print("Wappalyzer technologies directory does not appear to be available.")
+        return
+
+    dTechnologies = dict()
+    #Loop through each of the Wappalyzer technologies files.
+    for TechFileRef in rTechFolder:
+        rTechFile = None
+        if TechFileRef['name'].endswith('.json'):
+            try:
+                TechFileRef2 = requests.get(TechFileRef['url'], timeout=5).json()
+                sTechFileURL = TechFileRef2['download_url']
+                rTechFile = requests.get(sTechFileURL).json() 
+            except:
+                print("Wappalyzer technologies json file %s seen in directory not available." % TechFileRef['url'])
+            if rTechFile:
+                for name, details in rTechFile.items():
+                    dTechnologies[name] = details
     try:
         cursor.execute("DROP TABLE IF EXISTS wapp_technologies")
         cursor.execute("CREATE TABLE IF NOT EXISTS wapp_technologies (name VARCHAR(80), details VARCHAR(4096), " \
                        "PRIMARY KEY (name))")
-        dWappalyzer = json.loads(sWappalyzerResponse) 
-        dTechnologies = dWappalyzer.get('technologies')
         sInsertWappTech = """INSERT IGNORE INTO wapp_technologies (name, details) VALUES (%s,%s)""" 
         for sName in dTechnologies:
-            cursor.execute(sInsertWappTech, (sName.rstrip(),str(dTechnologies.get(sName)).rstrip()) ) 
+            params = (sName, json.dumps(dTechnologies.get(sName)))
+            cursor.execute(sInsertWappTech, params=params) 
         mysqlconn.commit()
     except Exception as e:
         print("Error inserting Wappalyzer data: %s" % e)
@@ -1036,30 +1148,33 @@ def search_fingerprint(sFingerprintName):
 def search_wappalyzer(sSearchName):
     TupleToRegexCheck = ()
     ListToRegexCheck = []
-
-    sSQL_Search_Technologies = "SELECT details FROM wapp_technologies WHERE name like %s"
+    Technologies_records = None
+    # sSQL_Search_Technologies = "SELECT details FROM wapp_technologies WHERE name LIKE %s"
+    sSQL_Search_Technologies = "SELECT details FROM wapp_technologies WHERE name = %s"
+    params = (sSearchName,)
     try:
-        cursor.execute(sSQL_Search_Technologies, ("%"+sSearchName+'%',) )
-        Technologies_records = cursor.fetchall()
+        cursor.execute(sSQL_Search_Technologies, params=params)
+        Technologies_record = cursor.fetchone()
     except Exception as e:
-        print("Error searching Wappalyzer tachnologies: %s" % e)
+        print(f"Error searching Wappalyzer technologies: {e}")
         return
-    if len(Technologies_records) > 0:
-        p = re.compile('(?<!\\\\)\'')
-        sWappalyzerTech = Technologies_records[0][0].replace("\"", "\\\"")
-        sWappalyzerTech = sWappalyzerTech.replace("\\'","\\\\'")
-        sWappalyzerTech = p.sub('\"', sWappalyzerTech)
-        sWappalyzerTech = re.sub(r'":\ (True|False),', '": "",', sWappalyzerTech)
-        dWappalyzerTech = json.loads(sWappalyzerTech)
+    if Technologies_record:
+        dWappalyzerTech = json.loads(Technologies_record[0])
 
         #For meta in wappalyzer technologies, they are regex but we have to format each one we get.
-        #We'll do that and add directly to the tuple now. 
+        #We'll do that and add directly to the tuple now.
         if "meta" in dWappalyzerTech.keys():
             if isinstance(dWappalyzerTech.get("meta"), dict):
                 for key in dWappalyzerTech["meta"]:
-                    if ";confidence:" not in dWappalyzerTech["meta"][key]:
-                        ListToRegexCheck.append("<meta name=[\"\']%s[\"\'] content=[\"\']%s" % (key, \
-                                                dWappalyzerTech["meta"][key].strip('^').strip('$')))
+                    values = list()
+                    if isinstance(dWappalyzerTech["meta"][key], str):
+                        values.append(dWappalyzerTech["meta"][key])
+                    else:
+                        values.extend(dWappalyzerTech["meta"][key])
+                    for value in values:
+                        if ";confidence:" not in value:
+                            ListToRegexCheck.append("<meta name=[\"\']%s[\"\'] content=[\"\']%s" % (key, \
+                                                value.strip('^').strip('$')))
         if "html" in dWappalyzerTech.keys():
             if isinstance(dWappalyzerTech.get("html"), str):
                 if ";confidence:" not in dWappalyzerTech.get("html"):
@@ -1139,7 +1254,7 @@ def url_request(tTarget):
         try:
             timeout = Timeout(8)
             timeout.start()
-            r = requests.get(sURL,  verify=False, timeout=3)
+            r = requests.get(sURL, verify=False, timeout=6)
         except Timeout as T:
             print("***Python requests.get() hung for URL %s." % sURL)
             return
@@ -1164,7 +1279,7 @@ def find_list_resources(tag, attribute, soup):
     return list
 
 
-def dns_zone_xfer(bUseFileOnly):
+def dns_zone_xfer(sTSIGfile):
     print("Attempting zone transfer for domains in database.")
     sZoneXfersQuery = "SELECT domains.domain as zone, domains.server as server, domains.keyname as keyname, " \
                       "tsig.algorithm as algorithm, tsig.secret FROM domains INNER JOIN tsig ON domains.keyname = tsig.name"
@@ -1201,14 +1316,14 @@ def dns_zone_xfer(bUseFileOnly):
     for tZoneInfo in lZoneInfo:
         sZone = tZoneInfo[0]
         sServer = tZoneInfo[1]
-        if bUseFileOnly == False:
+        if sTSIGfile == None:
             sKeyName = tZoneInfo[2]
             sAlgorithm = tZoneInfo[3]
             sSecret = tZoneInfo[4]
         else:
             #If the file-based tsig has been specified, take the values out of the file.
             try:
-                TSIGFile = open('./ws-key.tsig', 'r')
+                TSIGFile = open(sTSIGfile, 'r')
                 aTSIGLines = TSIGFile.readlines()
             except Exception as e:
                 print("Error opening TSIG file %s: %s" % (sZone, e))
@@ -1376,6 +1491,65 @@ def is_valid_ipv4(s):
         return False
 
 
+def ListOutsideNamesAndIPs():
+    lIntRanges = [] #Will be used to store tuples of int representations of net and mask for each range, better performance
+    lExtraAddresses = []
+    lAddrAndNamesToPrint = []
+
+    #Get names and IPs that came up in our zone transfer that are outside the ranges
+    sSQL_SelectDistinctIP = "SELECT distinct ip from dns_hosts"
+    sSQL_SelectNameAndIP  = "SELECT fqdn,ip from dns_hosts"
+
+    try:
+        cursor.execute(sSQL_SelectDistinctIP)
+        lIPs = cursor.fetchall()
+        cursor.execute(sSQL_SelectNameAndIP)
+        lAllDNS = cursor.fetchall()
+    except Exception as e:
+        print("Exception encountered while retrieving DNS hosts from database: %s", e)
+        return
+    sSQL_SelectRanges = "SELECT cidr from target_ranges"
+    try:
+        cursor.execute(sSQL_SelectRanges)
+        lRanges = cursor.fetchall()
+    except Exception as e:
+        print("Exception encountered while retrieving CIDR ranges from database: %s", e)
+        return
+    lTargetRanges = [] #Clean,just string,no tuple
+    for tCIDRblock in lRanges:
+        lTargetRanges.append(tCIDRblock[0])
+        ip_net = ipaddress.ip_network(tCIDRblock[0])
+        iNetw = int(ip_net.network_address)
+        iMask = int(ip_net.netmask)
+        lIntRanges.append((iNetw,iMask))
+
+    for address in lIPs:
+        iAddress = int(ipaddress.ip_address(address[0]))
+        bInRange = False
+        for tIntRange in lIntRanges:
+            bInRange = (iAddress & tIntRange[1]) == tIntRange[0]
+            if bInRange == True:
+                break
+        if bInRange == False:
+            lExtraAddresses.append(address[0])
+    lExtraAddresses = list(set(lExtraAddresses)) #Only Unique
+
+    #Now take our list against every row in the dns_hosts table.  Print [name],[IP] when the IP matches
+    dAllDNS = {}
+    for NameAndIP in lAllDNS:
+        if NameAndIP[0] not in dAllDNS: 
+            dAllDNS[NameAndIP[1]] = NameAndIP[0]
+        else:
+            dAllDNS[NameAndIP[1]].append(NameAndIP[0])
+    for sIP in lExtraAddresses:
+        if sIP in dAllDNS:
+            lAddrAndNamesToPrint.append("%s,%s" % (sIP,dAllDNS[sIP]))
+    lAddrAndNamesToPrint.sort()
+    for sAddrAndName in lAddrAndNamesToPrint:
+        print(sAddrAndName)
+    return
+
+
 def is_valid_hostname(hostname):
     if len(hostname) > 255:
         return False
@@ -1386,83 +1560,11 @@ def is_valid_hostname(hostname):
 
 
 def main():
-    parser = argparse.ArgumentParser()
-    parser.add_argument("--ADD-HTTP-PORT", "-a", dest="HttpPortToAdd", default=None, help="Add a custom HTTP port.")
-    parser.add_argument("--CLEAR-HTTP", "-aC", dest="ClearHttpPorts", default=False, action="store_true", \
-                        help="Clear any custom HTTP ports and revert to default of 80.")
-    parser.add_argument("--ADD-HTTPS-PORT", "-b", dest="HttpsPortToAdd", default=None, help="Add a custom HTTPS port.")
-    parser.add_argument("--CLEAR-HTTPS", "-bC", dest="ClearHttpsPorts", default=False, action="store_true", \
-                        help="Clear any custom HTTPS ports and revert to default of 443.")
-    parser.add_argument("--ADD-CUSTOM-FINGERPRINT", "-c", dest="Fingerprint", default=None, \
-                        help="Add a custom fingerprint in the form <Name>,<RegEx>.")
-    parser.add_argument("--DELETE-CUSTOM-FINGERPRINT", "-cD", dest="FingerprintNameToDelete", default=None, \
-                        help="Delete a custom fingerprint by name.")
-    parser.add_argument("--IMPORT-CUSTOM-FINGERPRINT", "-cI", dest="ImportFingerprintFile", default=None, \
-                        help="Import a custom fingerprint file with the path specified.")
-    parser.add_argument("--CLEAR-CUSTOM-FINGERPRINTS", "-cC", dest="ClearFingerprints", default=False, action="store_true", \
-                        help="Clears all custom fingerprints stored in DB.")
-    parser.add_argument("--SHOW-CONFIG", "-g", dest="ShowConfigBrief", default=False, action="store_true", \
-                        help="Show current WebStor configuration (brief).")
-    parser.add_argument("--SHOW-CONFIG-FULL", "-gF", dest="ShowConfigFull", default=False, action="store_true", \
-                        help="Show current WebStor configuration (full).")
-    parser.add_argument("--RUN-MASSCAN", "-m", dest='ForceScan', default=False, action='store_true', \
-                        help="Runs a new port scan with Masscan on all configured TCP ports for HTTP and HTTPS, " \
-                        "against all configured ranges and any IP addresses from DNS records that are outside those ranges.")
-    parser.add_argument("--SET-MASSCAN-RANGES", "-mR", dest="SetScanRanges", default=None, \
-                        help="Scan range or ranges, replaces existing ranges in DB, comma " \
-                        "separated, such as: -s 10.10.0.0/16,10.13.0.0/16,192.168.1.0/24")
-    parser.add_argument("--IMPORT-MASSCAN-RANGES", "-mI", dest="ImportScanRanges", default=None, \
-                        help="Import scan ranges (CIDR blocks) from a specified file.")
-    parser.add_argument("--DELETE-RANGE", "-mD", dest="RangeToDelete", default=None, help="Delete scan range.")
-    parser.add_argument("--IMPORT-ZONE-FILE", "-zI", dest="ImportZoneFile", default=None, \
-                        help="Add domains for zone transfers from a file.")
-    parser.add_argument("--ADD-PATH", "-p", dest="PathToAdd", default=None, help="Add paths for which to request " \
-                        "and store responses besides '/'.")
-    parser.add_argument("--DELETE-PATH", "-pD", dest="PathToDelete", default=None, help="Delete paths for which to " \
-                        "request and store responses besides '/'.")
-    parser.add_argument("--CLEAR-PATHS", "-pC", dest="ClearPaths", default=False, action="store_true", \
-                        help="Clear any custom URL request paths and revert to default of '/'.")
-    parser.add_argument("--REFRESH-RESPONSES", "-r", dest="RefreshResponses", default=False, action="store_true", \
-                        help="Refresh URL responses in DB.")
-    parser.add_argument("--SEARCH-PATTERN", "-sP", dest="SearchPattern", default=None, \
-                        help="Search for string or regular expression in WebStor database.")
-    parser.add_argument("--SEARCH-CUSTOM-FINGERPRINT", "-sC", dest="SearchFingerprint", default=None, \
-                        help="Search for technology by name of user-provided custom fingerprint.")
-    parser.add_argument("--SEARCH-WAPPALYZER", "-sW", dest="SearchWappalyzer", default=None, \
-                        help="Search for technology by name (from Wappalyzer Tech DB) in WebStor DB.")
-    parser.add_argument("--NO-TSIG-KEY", "-tN", dest="UseTSIG", default=True, action="store_false", \
-                        help="Do not use DNSSec TSIG key stored in database or a file, even if present.")
-    parser.add_argument("--TSIG-KEY-IMPORT", "-tI", dest="ImportTSIGFile", default=None, \
-                        help="Import a specified TSIG key file into the database")
-    parser.add_argument("--TSIG-KEY-REPLACE", "-tR", dest="ReplacementTSIGFile", default=None, \
-                        help="Replace a TSIG key in the database with a specified file")
-    parser.add_argument("--DELETE-TSIG", "-dT", dest="TSIGToDelete", default=None, \
-                        help="Delete a TSIG key from the database by name.")
-    parser.add_argument("--USE-TSIG-FILE-ONLY", "-tF", dest="UseTSIGFileOnly", default=False, action='store_true', \
-                        help="Only use tsig file 'webstor.tsig' in present directory, do not use TSIGs stored in the DB. " \
-                        "Applies to all domains, limiting WebStor to one TSIG for zone transfers in the current execution.")
-    parser.add_argument("--DOWNLOAD-NEW-WAPPALYZER", '-w', dest="DLWap", default=False, action="store_true", \
-                        help="Download a new Wappalyzer fingerprints file directly from GitHub. Overwrites existing " \
-                        "Wappalyzer fingerprint data.")
-    parser.add_argument("--LIST-WAPPALYZER-TECH-NAMES", "-wL", dest="ListWappalyzer", default=False, action="store_true", \
-                        help="List the names of all Wappalyzer technologies in the database.")
-    parser.add_argument("--ZONE-XFER", "-z", dest='PerformZoneXfer', default=False, action='store_true', \
-                        help="Forces a new zone transfer using all domains, servers, and associated TSIG keys in DB")
-    parser.add_argument("--ADD-DOMAIN", "-zA", dest="DomainDetails", default=None, \
-                        help="Add a domain in the form <Domain name>,<Server>,<TSIG Key Name>.")
-    parser.add_argument("--DELETE-DOMAIN", "-zD", dest="DomainToDelete", default=None, \
-                        help="Delete a DNS domain from the database by name.")
-    parser.add_argument("--CLEAR-DOMAINS", "-zC", dest="ClearDomains", default=False, action="store_true", \
-                        help="Clears all DNS domains stored in DB.")
-    parser.add_argument("--LIST-DOMAINS", "-zL", dest="ListDomains", default=False, action="store_true", \
-                        help="Lists all DNS domains stored in DB.")
 
     if len(sys.argv)==1:
         parser.print_help(sys.stderr)
         sys.exit(1)
 
-    args = parser.parse_args()
-
     create_database()
         
     if args.ClearHttpPorts != False:
@@ -1499,6 +1601,8 @@ def main():
         set_scan_ranges(args.SetScanRanges)
     if args.ImportScanRanges != None:
         import_scan_ranges(args.ImportScanRanges)        
+    if args.RangeToAdd != None:
+        add_scan_range(args.RangeToAdd)
     if args.DLWap != False:
         download_wappalyzer()
     if args.ListWappalyzer != False:
@@ -1529,9 +1633,11 @@ def main():
         search_fingerprint(args.SearchFingerprint)    
     if args.SearchWappalyzer != None:
         search_wappalyzer(args.SearchWappalyzer)    
-
+    if args.ListOutside != False:
+        ListOutsideNamesAndIPs()    
 
 start_time = time.time()
 
+
 if __name__ == '__main__':
     main()
